# TODO
# - Unbundle libraries (jansson/jemalloc/luajit/s2-geometry-library)
# - Also, aerospike-common/asmalloc/aerospike-lua-core/aerospike-mod-lua should be in their own packages
# - Use dynamically linked libraries (openssl/lua and all others above)
# - Disable telemetry agent by default
# - Check build flags 

%if 0%{?fedora} || 0%{?rhel} >= 7
%global use_systemd 1
%else
%global use_systemd 0
%endif

%global use_asm 0
%global use_telemetry_agent 0

# Building docs currently segfaults...
%global build_docs 0

#global asmalloc_githash            1560819
#global common_githash              c886540
#global jansson_githash             5cc594c
#global jemalloc_githash            46c0af6
#global lua-core_githash            2c910a1
#global luajit_githash              6c4a182
#global mod-lua_githash             06448f1
#global s2-geometry-library_githash c6e27a1
#global telemetry_githash           6d0bc9f

Name:    aerospike-server
Version: 3.9.0.2
Release: 1%{?dist}
Summary: Distributed, scalable NoSQL database

License: AGPLv3 and ASL 2.0
URL:     http://www.aerospike.com
Source0: https://github.com/aerospike/%{name}/archive/%{version}.tar.gz
Source1: https://github.com/aerospike/asmalloc/archive/1.0.0.tar.gz
Source2: https://github.com/aerospike/aerospike-common/archive/c886540f86a2bbdfdf3a34ce606823927d09d3a3.tar.gz
Source3: https://github.com/aerospike/jansson/archive/5cc594c9e8bc01f9531f80aba82c9775bba94c18.tar.gz
Source4: https://github.com/aerospike/jemalloc/archive/3.6.0.tar.gz
Source5: https://github.com/aerospike/aerospike-lua-core/archive/2c910a16f6891b9b33f855cdc4a36aa3ffb69478.tar.gz
Source6: https://github.com/aerospike/luajit/archive/2.0.3.tar.gz
Source7: https://github.com/aerospike/aerospike-mod-lua/archive/4de3e119e6212bf693d0c8951906172f71191035.tar.gz
Source8: https://github.com/aerospike/s2-geometry-library/archive/c6e27a1789fa5fac27ddd3d0a54d80c9e149cfdb.tar.gz
Source9: https://github.com/aerospike/aerospike-telemetry-agent/archive/1.0.4.tar.gz

Patch0:  aerospike-server-3.9.0.2-Allow_use_of_system_libraries.patch

BuildRequires: git 
BuildRequires: autoconf 
BuildRequires: automake 
BuildRequires: jansson-devel
BuildRequires: jemalloc-devel
BuildRequires: libtool
BuildRequires: libxslt
BuildRequires: lua-devel
#BuildRequires: luajit              # Bundled
BuildRequires: openssl-devel
#BuildRequires: s2-geometry-library # Bundled (Not packaged yet)
%if %{build_docs}
BuildRequires: doxygen
%endif
Requires: logrotate

%if %{use_systemd}
BuildRequires: systemd
%endif

%if ! %{use_systemd}
Requires(post):  chkconfig
Requires(preun): chkconfig
Requires(preun): initscripts
Requires(postun): initscripts
%endif

ExcludeArch: %{ix86}


%description
The Aerospike distributed datastore allows fully scalable
and reliable data storage with elastic server properties.
This package contains all of the code for running the Aerospike server.


%prep
%setup -q
# submodules
# - asmalloc            The ASMalloc Memory Allocation Tracking Tool
# - common              The Aerospike Common Library
# - lua-core            The Aerospike Core Lua Source Files
# - mod-lua             The Aerospike Lua Interface
# - telemetry           The Aerospike Telemetry Agent (Community Edition only)
# - jansson             C library for encoding, decoding and manipulating JSON data (http://www.digip.org/jansson/)
# - jemalloc            The JEMalloc Memory Allocator                               (http://www.canonware.com/jemalloc/)
# - luajit              The LuaJIT (Just-In-Time Compiler for Lua)                  (http://luajit.org/luajit.html)
# - s2-geometry-library The S2 Spherical Geometry Library                           (https://code.google.com/archive/p/s2-geometry-library/)

cd modules/
%setup -q -T -D -a 1
%setup -q -T -D -a 2
%setup -q -T -D -a 3
%setup -q -T -D -a 4
%setup -q -T -D -a 5
%setup -q -T -D -a 6
%setup -q -T -D -a 7
%setup -q -T -D -a 8
%setup -q -T -D -a 9

rm -r modules/*
mv asmalloc*                  modules/asmalloc
mv aerospike-common*          modules/common
mv jansson*                   modules/jansson
mv jemalloc*                  modules/jemalloc
mv aerospike-lua-core*        modules/lua-core
mv luajit*                    modules/luajit
mv aerospike-mod-lua*         modules/mod-lua
mv s2-geometry-library*       modules/s2-geometry-library
mv aerospike-telemetry-agent* modules/telemetry

# Fix init scripts
sed -i -e 's/# chkconfig: 2345 85 15/# chkconfig: - 85 15/' as/etc/init-script
sed -i -e '/# Default-Start:/d' as/etc/init-script
sed -i -e 's!LOCKFILE=/var/lock/subsys/aerospike!LOCKFILE=/var/lock/subsys/aerospike-server!' \
    as/etc/init-script
#aerospike-server.x86_64: W: no-reload-entry /etc/rc.d/init.d/aerospike

%if %{use_telemetry_agent}
sed -i -e 's/# chkconfig: 2345 85 15/# chkconfig: - 85 15/' as/etc/init-telemetry-script
sed -i -e 's/# Deafult-Stop:/# Default-Stop:/' as/etc/init-telemetry-script
sed -i -e '/# Default-Start:/d' as/etc/init-telemetry-script
#aerospike-server.x86_64: W: no-reload-entry /etc/rc.d/init.d/aerospike_telemetry
#aerospike-server.x86_64: E: subsys-not-used /etc/rc.d/init.d/aerospike_telemetry
%endif

# Use system libs
%patch0 -p1

# Don't use /opt/aerospike, use /var/lib/aerospike instead
sed -i -e 's!/opt/aerospike!/var/lib/aerospike!' as/src/base/cfg.c
sed -i -e 's!/opt/aerospike!/var/lib/aerospike!' modules/mod-lua/src/main/mod_lua.c


%build
make %{?_smp_mflags} server \
  LD_CRYPTO=dynamic \
  LD_LUA=dynamic \
  LD_JANSSON=system \
  LD_JEM=system \
  LD_LUAJIT=dynamic \
  USE_LUAJIT=0 \
  USE_ASM=0

%if %{build_docs}
pushd apidocs
make docs
popd
%endif

%install
install -d %{buildroot}/etc/aerospike
install -pm 644 as/etc/aerospike.conf %{buildroot}/etc/aerospike/aerospike.conf

%if %{use_systemd}
install -d %{buildroot}/etc/systemd/system/aerospike.service.d
%else
install -d %{buildroot}/var/log/aerospike
install -d %{buildroot}/var/run/aerospike
install -d %{buildroot}/%{_initrddir}
install -d %{buildroot}/etc/logrotate.d
%endif

install -d %{buildroot}/usr/bin
install -pm 755 target/Linux-x86_64/bin/asd %{buildroot}%{_bindir}/asd

%if %{use_systemd}
install -pm 755 tools/bin/asd-coldstart %{buildroot}%{_bindir}/asd-coldstart
%endif
install -pm 755 tools/citrus2aero/upgrade2to3 %{buildroot}%{_bindir}/asmigrate2to3
install -pm 755 tools/fixownership/fixownership.py %{buildroot}%{_bindir}/asfixownership

# Install sample confs
mkdir -p sample
install -pm 644 as/etc/README.sample.conf.md sample/README.md
install -pm 644 as/etc/aerospike_ssd.conf sample/aerospike_ssd.conf
install -pm 644 as/etc/aerospike_mesh.conf sample/aerospike_mesh.conf

%if %{use_systemd}
cat as/etc/aerospike.service.head >> pkg/rpm/aerospike.service
cat as/etc/aerospike.service.telemetry >> pkg/rpm/aerospike.service
cat as/etc/aerospike.service.tail >> pkg/rpm/aerospike.service
install -p -D -m 644 pkg/rpm/aerospike.service %{buildroot}%{_unitdir}/aerospike.service
install -p -D -m 644 as/etc/aerospike-server.tmpfiles %{buildroot}/etc/tmpfiles.d/aerospike.conf
install -p -D -m 644 as/etc/aerospike-server.sysconfig %{buildroot}/etc/sysconfig/aerospike
install -p -D -m 755 as/etc/asd-systemd-helper %{buildroot}%{_bindir}/asd-systemd-helper
install -p -D -m 644 as/etc/aerospike.service.d/* %{buildroot}/etc/systemd/system/aerospike.service.d
%else
install -pm 755 as/etc/init-script %{buildroot}%{_initrddir}/aerospike-server
sed -i 's/@EDITION@/Community/g' %{buildroot}%{_initrddir}/aerospike-server
install -pm 644 as/etc/logrotate_asd %{buildroot}/etc/logrotate.d/aerospike-server
%endif

install -d %{buildroot}%{_localstatedir}/lib/aerospike/



install -d %{buildroot}%{_localstatedir}/lib/aerospike/data
install -d %{buildroot}%{_localstatedir}/lib/aerospike/smd
install -d %{buildroot}%{_localstatedir}/lib/aerospike/sys/udf/lua/external
install -d %{buildroot}%{_localstatedir}/lib/aerospike/sys/udf/lua/ldt
install -d %{buildroot}%{_localstatedir}/lib/aerospike/usr/udf/lua
install -pm 644 modules/lua-core/src/*.lua %{buildroot}%{_localstatedir}/lib/aerospike/sys/udf/lua
install -pm 644 modules/lua-core/src/external/* %{buildroot}%{_localstatedir}/lib/aerospike/sys/udf/lua/external
install -pm 644 modules/lua-core/src/ldt/* %{buildroot}%{_localstatedir}/lib/aerospike/sys/udf/lua/ldt

%if %{use_telemetry_agent}
install -d %{buildroot}%{_localstatedir}/lib/aerospike/telemetry
install -d %{buildroot}%{_localstatedir}/lib/aerospike/telemetry/phonehome
install -d %{buildroot}%{_localstatedir}/lib/aerospike/telemetry/daemon
install -pm 644 as/etc/telemetry.conf %{buildroot}/etc/aerospike
install -pm 755 modules/telemetry/telemetry.py %{buildroot}%{_localstatedir}/lib/aerospike/telemetry
install -pm 755 modules/telemetry/phonehome/*.py %{buildroot}%{_localstatedir}/lib/aerospike/telemetry/phonehome
install -pm 755 modules/telemetry/daemon/*.py %{buildroot}%{_localstatedir}/lib/aerospike/telemetry/daemon
%if %{use_systemd}
install -pm 755 as/etc/aerospike_telemetry.service %{buildroot}%{_unitdir}/aerospike_telemetry.service
install -pm 644 as/etc/aerospike_telemetry.sysconfig %{buildroot}/etc/sysconfig/aerospike_telemetry
%else
install -pm 755 as/etc/init-telemetry-script %{buildroot}/%{_initrddir}/aerospike-telemetry-agent
install -pm 644 as/etc/logrotate_telemetry %{buildroot}/etc/logrotate.d/aerospike-telemetry-agent
%endif
%endif

%if %{use_asm}
install -pm 644 /etc/initfns %{buildroot}/etc/aerospike
install -d %{buildroot}%{_localstatedir}/lib/aerospike//{bin,lib}
install -pm 755 as/etc/run-under-gdb %{buildroot}%{_localstatedir}/lib/aerospike//bin
install -pm 644 modules/asmalloc/lib/asmalloc.jem.so %{buildroot}%{_localstatedir}/lib/aerospike/lib
install -pm 644 modules/jemalloc/lib/libjemalloc.so %{buildroot}%{_localstatedir}/lib/aerospike/lib
%endif


%pre
getent group aerospike >/dev/null || groupadd -r aerospike
getent passwd aerospike >/dev/null || \
  /usr/sbin/useradd -r \
      -d "/var/lib/aerospike" \
      -c "System User for Aerospike" \
      -s "/sbin/nologin" \
      -g aerospike \
      aerospike
exit 0

%post
%if %{use_systemd}
%systemd_post aerospike.service
%if %{use_telemetry_agent}
%systemd_post aerospike_telemetry.service
%endif
%else
/sbin/chkconfig --add aerospike-server
%if %{use_telemetry_agent}
/sbin/chkconfig --add aerospike-telemetry-agent
%endif
%endif

%preun
%if %{use_systemd}
%systemd_preun aerospike
%if %{use_telemetry_agent}
%systemd_preun aerospike-telemetry-agent
%endif
%else
if [ $1 -eq 0 ] ; then
    /sbin/service aerospike-server stop >/dev/null 2>&1
    /sbin/chkconfig --del aerospike-server
%if %{use_telemetry_agent}
    /sbin/service aerospike-telemetry-agent stop >/dev/null 2>&1
    /sbin/chkconfig --del aerospike-telemetry-agent
%endif
fi
%endif

%postun
%if %{use_systemd}
%systemd_postun_with_restart aerospike.service
%if %{use_telemetry_agent}
%systemd_postun_with_restart aerospike_telemetry.service
%endif
%else
/sbin/service aerospike-server condrestart
%if %{use_telemetry_agent}
/sbin/service aerospike-telemetry-agent condrestart
%endif
%endif


%files
%license LICENSE LICENSE-AGPL LICENSE-APACHE
%doc README.md sample/
%{_bindir}/asd
%{_bindir}/asmigrate2to3
%{_bindir}/asfixownership
%config(noreplace) %{_sysconfdir}/aerospike/aerospike.conf
%if %{use_systemd}
%{_unitdir}/aerospike.service
%else
%{_initrddir}/aerospike-server
%endif
%config(noreplace) %{_sysconfdir}/logrotate.d/aerospike-server
%{_localstatedir}/log/aerospike/
%defattr(-,aerospike,aerospike,-)
/var/lib/aerospike

%if %{use_telemetry_agent}
%config(noreplace) %{_sysconfdir}/aerospike/telemetry.conf
%if %{use_systemd}
%{_unitdir}/aerospike_telemetry.service
%else
%{_initrddir}/aerospike-telemetry-agent
%endif
%config(noreplace) %{_sysconfdir}/logrotate.d/aerospike-telemetry-agent
%endif


%changelog
* Thu Aug 04 2016 Xavier Bachelot <xavier@bachelot.org> 3.9.0.2-1
- Update to 3.9.0.2.

* Wed Jun 08 2016 Xavier Bachelot <xavier@bachelot.org> 3.8.4-1
- Initial build.

